#!/usr/bin/python

import os
import sys
import string
import commands
import subprocess
import signal
import re
from time import sleep
import gmenu

class commandline:
    def __init__(self):
        self.plugin_list = []
        if os.getuid() != 0:
            return False

    def shell_exec(self, cmd, wait = True):
        return_text = ''
        cmd=['export LC_ALL=C;'+cmd]
        #print cmd
        return_shell = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        if wait == True:
            return_text = return_shell.stdout.readlines()
            return_shell.wait()
            if not return_shell.returncode == 0:
                return False
        return return_text

    def shell_exec_for_user(self, cmd, user, wait = True):
        userinfolist = self.poll_userlist(user)
        if not userinfolist:
            return False
        for userinfo in userinfolist:
            user = userinfo[1]
            display = userinfo[3]
            home = userinfo[4]
            xauth = userinfo[5]
            if xauth == None:
                 xauthcmd=';export XAUTHORITY='+home+'/.Xauthority'
            else:
                xauthcmd=';export XAUTHORITY='+xauth
            dbusaddr = self.read_proc_environ_file(self.poll_proclist([user], ['metacity']), 'DBUS_SESSION_BUS_ADDRESS')
            if dbusaddr:
                dbuscmd=';export '+dbusaddr

            shcmd = ('su - '+user+' -c "export LC_ALL=C;export DISPLAY='+display+xauthcmd+dbuscmd+';'+cmd+'"')
            return_exec = self.shell_exec(shcmd, wait)
            if return_exec == False:
                print "shell_exec_for_user: error"
                return False
        return return_exec

    def test_user(self, user):
        if not re.match('^[\w\.\-]+$',user):
            print "Utilisateur match pas"
            return False
        if not self.shell_exec('id '+user):
            return False
        return True

    def poll_userlist(self, users = None, groups = False, ip = None):
        """
        Returns a matrix of pid and user@host for open inbound ssh based LTSP sessions:
        ([pid0], [user0], [host0], [display0], [home0]), ([pid1], [user1], [host1], [display1], [home1]), ...)
        """
        host=''
        display=''
        home=''
        if not users == None:
            if type(users) != list:
                print "poll_userlist: users need to be a list"
                return False
            for user in users:
                if not self.test_user(user):
                    return False
        if not ip == None:
            if type(ip) != str:
                print "poll_userlist: ip need to be a str"
                return False
            if users != None:
                print "poll_userlist: users and ip not allowed together"
                return False
        ulist = []
        cmd = 'ps --no-headers eo pid,ppid,command -C x-session-manager'
        clist = self.shell_exec(cmd)
        if clist != False:
            for line in clist:
                pid = line.split()[0]
                ppid = line.split()[1]
                #for gdm
                cmd = 'ps --no-headers eo command -p %s' % ppid
                tpid = self.shell_exec(cmd)
                if tpid[0].split()[0] == "/usr/sbin/gdm":
                    cpid=pid
                else:
                    cpid=ppid
                user = None
                host = None
                display = None
                home = None
                xauth = None
                for elem in line.split():
                    if elem.startswith('USER='):
                        user = elem.lstrip('USER=')
                    elif elem.startswith('SSH_CLIENT='):
                        host = elem.lstrip('SSH_CLIENT=')
                    elif elem.startswith('DISPLAY='):
                        display = elem.lstrip('DISPLAY=')
                    elif elem.startswith('HOME='):
                        home = elem.lstrip('HOME=')
                    elif elem.startswith('XAUTHORITY='):
                        xauth = elem.lstrip('XAUTHORITY=')
                if ip != None:
                    if host == ip:
                        return user
                elif not user == None:
                    if users == None or user in users:
                        if host == None:
                            host='localhost'
                        if display != None and home != None:
                            if groups == False:
                                ulist.append([cpid, user, host, display, home, xauth])
                            else:
                                cmd = 'id -nG '+user
                                groups = self.shell_exec(cmd)
                                groups = string.join(groups[0].rstrip().strip().split(), ':')
                                ulist.append([cpid, user, host, display, home, xauth, groups])
        if ulist == []:
            return ''
        return ulist

    def poll_proclist(self, users = None, filter = None):
        """
        return the users procs
        """
        if users != None:
            if not type(users) == list:
                print "poll_proclist: users need to be a list"
                return False
            for user in users:
                if not self.test_user(user):
                    return False
            users = string.join(users, ',')

        if filter != None:
            if not type(filter) == list:
                print "filter need to be a list"
                return False
            filter = string.join(filter, ',')
        procs = []
        cmd = 'ps --no-headers -o '
        if users != None and filter == None:
            cmd = cmd + 'pid,%mem,%cpu,nice,cmd -U '+users
        elif filter != None:
            if users != None:
                cmd = cmd+'pid,'
            cmd = cmd + 'user -C '+filter
#            if users != None:
#                cmd = cmd+'| grep "^'+user+'$"'
        else:
                        print "poll_proclist: set filter or user please"
                        return False
        proclist = self.shell_exec(cmd)
        if proclist == [] or proclist == False:
            return False
        if filter == None:
            for proc in proclist:
                comm = proc.rstrip().split()[4]
                if comm.split('/')[-1] == "python":
                    if len(proc.rstrip().split()) > 5:
                        comm = proc.rstrip().split()[5]
                elif comm == "bash":
                    if len(proc.rstrip().split()) > 8 and proc.rstrip().split()[8]:
                            comm = None
                    elif len(proc.rstrip().split()) == 6 and proc.rstrip().split()[6] == LTSPROCKS:
                            comm = None
                elif comm == '/bin/sh' and proc.rstrip().split()[5] and proc.rstrip().split()[5] == '-':
                    comm = None
                if not comm == None \
                    and not comm == 'grep' \
                    and not comm == 'su' \
                    and not comm == '[sh]' \
                    and not comm == '-su' \
                    and not comm == '-bash' \
                    and not comm == 'ps' \
                    and not comm == 'ltspfs' \
                    and not comm == 'gnome-pty-helper' \
                    and not comm == '/usr/lib/gvfs/gvfsd' \
                    and not comm == '/usr/lib/gvfs/gvfsd-trash' \
                    and not comm == '/usr/lib/gvfs/gvfsd-burn' \
                    and not comm == '/usr/lib/gnome-applets/trashapplet' \
                    and not comm == 'gnome-power-manager' \
                    and not comm == '/usr/lib/bonobo-activation/bonobo-activation-server' \
                    and not comm == 'dbus-daemon' \
                    and not comm == 'dbus-launch' \
                    and not comm == '/usr/bin/dbus-daemon' \
                    and not comm == '/usr/bin/ck-launch-session' \
                    and not comm == '/usr/lib/libgconf2-4/gconfd-2' \
                    and not comm == '/usr/bin/ssh-agent' \
                    and not comm == 'sshd:' \
                    and not comm == 'x-session-manager' \
                    and not comm == '/usr/bin/gnome-keyring-daemon' \
                    and not comm == 'gnome-settings-daemon' \
                    and not comm == 'metacity' \
                    and not comm == '/usr/bin/metacity' \
                    and not comm == 'nautilus' \
                    and not comm == 'gnome-screensaver' \
                    and not comm == 'gnome-panel' \
                    and not comm == '/usr/bin/seahorse-agent' \
                    and not comm == '/usr/bin/scp-client' \
                    and not comm == '/usr/lib/gnome-applets/mixer_applet2' \
                    and not comm == '/usr/lib/notification-daemon/notification-daemon' \
                    and not comm == 'update-notifier' \
                    and not comm == '/usr/share/system-config-printer/applet.py':
                    commandtmp=proc.rstrip().split()
                    command=[commandtmp[0],str(comm.split('/')[-1]),commandtmp[1],commandtmp[2],commandtmp[3]]
                    procs.append(command)

        elif users == None:
            for proc in proclist:
                procs.append([proc.rstrip().split()[0]])
        else:
            for proc in proclist:
                if proc.rstrip().split()[1] == user:
                    return proc.rstrip().split()[0]
            return False
        return procs

    def cmd_to_pid(self, cmd, user=False, ppid=False):
        if type(cmd) != str:
            return False
        command = 'ps --no-headers -o '
        if ppid == True:
            command = command+'p'
        command = command+'pid,user -C '+cmd
        listu = self.shell_exec(command)
        if listu == False:
            return False
        pidu = []
        for listuv in listu:
            listuv = listuv.rstrip().split()
            if user == False:
                pidu.append(listuv[0])
            else:
                if listuv[1] == user:
                    pidu.append(listuv[0])
        if pidu == []:
            return False
        return pidu

    def pid_to_user(self, pid):
        if type(pid) != int:
            return False
        listu = self.shell_exec('ps --no-headers -o user -p '+str(pid))
        return listu

    def read_proc_environ_file(self, upid, searchstring):
        for elem in file('/proc/%s/environ' % upid).read().splitlines()[0].split('\x00'):
            if elem.startswith(searchstring+'='):
                return elem
        return False

    def check_start_gnome_screensaver(self, user):

        user=[user]
        cmd='gnome-screensaver'
        upid = self.poll_proclist(user, [cmd])
        if upid == False:
            self.shell_exec_for_user(cmd, user, False)
            sleep(1)
            upid = self.poll_proclist(user, [cmd])
            if upid == False:
                return False
        return True

    def lock(self, users):
        if type(users) != list:
            print "lock: users need to be a list"
            return False
        for user in users:
            if not self.test_user(user):
                return False
            if self.check_start_gnome_screensaver(user) == False:
                return False
            return self.shell_exec_for_user('gnome-screensaver-command -a', [user], False)

    def unlock(self, users):
        if type(users) != list:
            print "unlock: users need to be a list"
            return False
        for user in users:
            if not self.test_user(user):
                return False
            if self.check_start_gnome_screensaver(user) == False:
                return False
            return self.shell_exec_for_user('gnome-screensaver-command -d', [user])

    def get_lock(self, user):
        if type(user) != list:
            print "get_lock: user need to be a list"
            return False
        if not len(user) == 1:
            print 'only on user is allowed'
            return False
        if elf.check_start_gnome_screensaver(user[0]) == False:
            return False
        returncmd = self.shell_exec_for_user('gnome-screensaver-command -q', user)
        if returncmd == False:
            return False
        if not returncmd == []:
            returncmd = returncmd[0].split()
            if len(returncmd) == 4:
                if returncmd[3] == "active":
                    return True
        return False

    def kill_process(self, process, user = None):
        if re.match('^\d+$', process):
            if user == None:
                pidu = [process]
            else:
                useru = self.shell_exec('ps --no-headers -o user -p '+process+'')
                if useru == [user+"\n"]:
                     pidu = [process]
                else:
                    return False
        else:
            if user == None:
                pidu = self.cmd_to_pid(process)
            else:
                if self.test_user(user):
                    pidu = self.cmd_to_pid(process, user)
                else:
                    return False
            if pidu == False:
                return False
        for piduv in pidu:
            cmd = 'kill -15 '+piduv
            self.shell_exec(cmd)

    def start_process(self, users, command):
        command = 'export LC_ALL=fr_FR.UTF-8; ' + command
        if users == None:
            return self.shell_exec(command)
        if type(users) != list:
            print "start_process: users need to be a list"
            return False
        return self.shell_exec_for_user(command, users, False)

    def send_message(self, users, message, title):
        if title == None:
            title='Message from admin'
        cmd = "/usr/share/eclair-client-manager/message.py -t '" + title + "' -m '" + message + "'"
#        cmd = "zenity --warning --text='" + message + "' --title='" + title + "'"
        #print cmd
        return self.shell_exec_for_user(cmd, users, False)

    def logout_user(self, users=False):
        if users == False:
            users = []
            userstmp = self.poll_userlist()
            if not userstmp:
                return False
            for user in userstmp:
                users.append(user[1])
        if type(users) != list:
            print "logout_user: users need to be a list"
            return False
        for user in users:
            #If user is a number we assume that user is a pid number
            #so we search pid associate to this pid and try to find
            #all nautilus launch by this user. We search also if
            #nautilus is associate to the pid number
            if re.match('^\d+$', user):
                initpid = user
                user = self.pid_to_user(int(user))
                if user == False:
                    return False
                user = user[0].split('\n')[0]
                pids = self.cmd_to_pid('nautilus', user)
                if pids == False:
                    return False
                vals = []
                for pid in pids:
                    ppidcmd='ps --no-header eo ppid -p '
                    ppid = pid
                    for i in xrange(4):
                        ppid = self.shell_exec(ppidcmd+ppid)
                        if ppid == False:
                            return False
                        ppid = ppid[0].split('\n')[0].strip()
                        if initpid == ppid:
                            vals.append(pid)
                if vals == []:
                    return False
            else:
                if self.test_user(user):
                    vals = self.cmd_to_pid('nautilus', user)
                else:
                    print "Unknown user"
                    return False
            if vals == False:
                print "Unknown user"
                return False
            for val in vals:
                cmdenv='export '+self.read_proc_environ_file(val, 'SESSION_MANAGER')
                cmd = cmdenv+';gnome-session-save --silent --kill'
                self.shell_exec_for_user(cmd, [user], False)

    #---- return gnome menu
    def get_command_menu(self, entries):
        for entry in entries:
            if entry.get_type() == gmenu.TYPE_ENTRY and not entry.is_excluded:
                entry_exec = entry.get_exec().strip()
                entry_exec = entry_exec.replace('%u', '')
                entry_exec = entry_exec.replace('%U', '')
                yield (entry.get_icon(), entry.get_name(), entry_exec)
            elif entry.get_type() == gmenu.TYPE_DIRECTORY:
                yield self.get_menu(entry.get_contents(), valid_name=(entry.get_name(), entry.get_icon()))


    def get_menu(self, entries, valid_name=False):
        """
        entries: the list of menu entries
        valid_name: the parent directory
        """
        result = {}
        for entry in entries:

            if entry.get_type() == gmenu.TYPE_DIRECTORY:
                name = entry.get_name()
                icon = entry.get_icon()
                result[(name, icon)] = list(self.get_command_menu(entry.get_contents()))

            elif not valid_name==False:
                result[valid_name]= list(self.get_command_menu([entry]))

            elif not entry.get_type() == gmenu.TYPE_SEPARATOR:
                if not result.has_key('other'):
                    result[('__other__', None)] = []
                result[('__other__', None)].append(list(self.get_command_menu([entry])))
        return result

    def get_gnome_menu(self):
        return self.get_menu(gmenu.lookup_tree('applications.menu').root.get_contents())

    def vncpasswd(self):
        return file('/etc/tcmpasswd.clear').read().splitlines()[0].strip()

    def cleanghost(self):
         os.system("/usr/share/eole/cleanghost.py")

    def execmachin(self, machin, mcmd):
        cmd = "ssh -o BatchMode=yes root@%s %s" % (machin, mcmd)
        return self.shell_exec(cmd)

    def list_machin(self, mcmd=None, attrib=None):
#        machin = self.shell_exec("grep ' nbdrootd\[' /var/log/daemon.log | grep \"`LC_ALL=C date +%'b %e'`\"| grep \"]: connect from \" | awk '{ print $8 }' | sort -u")
        textattrib=''
        if attrib:
            textattrib='"(%s=true)"' % attrib
        machin = self.shell_exec("/usr/bin/slptool findsrvs eclair %s|sed -ne 's@^service:eclair://\([^ ]\+\),65535@\\1@p'" % textattrib)
        listm=[]
        if machin == False:
            return False
        for ip in machin:
            ip = ip.strip().rstrip()
            if mcmd == None:
                listm.append(ip)
            else:
                returnexec = self.execmachin(ip, mcmd)
                if returnexec:
                    listm.append([ip, returnexec])
        return listm

    def halt_serv(self,user=False, machin=False):
        if user and machin:
            cmd = "echo \"/usr/share/eole/eclairhalt.sh %s %s\" | at now + \"1 minutes\"" % (user, machin)
        else:
            cmd = "echo \"/usr/share/eole/eclairhalt.sh\" | at now + \"1 minutes\""
        return_exec = self.shell_exec(cmd)
        if return_exec == False:
            return False
        message = "Votre machine va etre arrete. Veuillez enregistrer votre travail."
        title = "Attention"
        if not user:
            users = []
            userstmp = self.poll_userlist()
            if not userstmp:
                return False
            for user in userstmp:
                users.append(user[1])
        else:
            users = [user]
        self.send_message(users, message, title)

    def test_battery(self):
        listip = self.list_machin('battery', 'battery')
        if listip:
            for battery in listip:
                ip=battery[0]
                result=battery[1][0].split('\n')
                for bat in result:
                    if bat:
                        bat=bat.split()
                        state = bat[2][:-1]
                        if state == 'discharging':
                            pourcent = bat[3][:-2]
                            try:
                                pourcent=int(pourcent)
                            except:
                                continue
                            if pourcent < 20:
                                name = bat[1][:-1]
                                timeleft=bat[4]
                                user = self.poll_userlist(ip=ip)
                                message = "Attention, la batterie \"%s\" devient faible :\nCharge restante : %s\nTemps restant : %s" % (name, pourcent, timeleft)
                                title = "Alerte batterie"
                                self.send_message([user], message, title)
#        return False
